? wine/patch.diff
Index: wine/windows/defdlg.c
===================================================================
RCS file: /home/wine/wine/windows/defdlg.c,v
retrieving revision 1.31
diff -u -r1.31 defdlg.c
--- wine/windows/defdlg.c	20 Apr 2004 04:02:35 -0000	1.31
+++ wine/windows/defdlg.c	22 Jun 2004 21:42:08 -0000
@@ -131,28 +131,51 @@
  *
  * Set the new default button to be hwndNew.
  */
-static BOOL DEFDLG_SetDefButton( HWND hwndDlg, DIALOGINFO *dlgInfo, WPARAM wParam )
+static BOOL DEFDLG_SetDefButton( HWND hwndDlg, DIALOGINFO *dlgInfo, WPARAM wParam, BOOL bSetDefID)
 {
     DWORD dlgcode=0; /* initialize just to avoid a warning */
     HWND hwndNew = GetDlgItem(hwndDlg, wParam);
+    INT idOldDefPushButton = dlgInfo->idResult;
 
-    dlgInfo->idResult = wParam;
+    /* Set default button ID only if bSetDefID is set */
+    if( bSetDefID )
+        dlgInfo->idResult = wParam;
+    
     if (hwndNew &&
         !((dlgcode=SendMessageW(hwndNew, WM_GETDLGCODE, 0, 0 ))
-            & (DLGC_UNDEFPUSHBUTTON | DLGC_BUTTON)))
-        return FALSE;  /* Destination is not a push button */
+            & (DLGC_UNDEFPUSHBUTTON | DLGC_BUTTON)))
+    {
+        if( bSetDefID )
+            return FALSE;
 
-    if (dlgInfo->idResult)  /* There's already a default pushbutton */
+        /**
+         * If bSetDefID is not set means, need to draw only default push button 
+         * rectangle.
+         * 
+         * Since the next control is not a push button, need to draw the push 
+         * button rectangle for the default control.
+         */
+        hwndNew = GetDlgItem(hwndDlg, dlgInfo->idResult);
+        dlgcode=SendMessageW(hwndNew, WM_GETDLGCODE, 0, 0 );
+    } /* ENDIF: Destination is not a push button */
+        
+    /** Make sure the old default control is a valid push button ID */
+    if( !idOldDefPushButton ||
+        !(SendMessageA(GetDlgItem(hwndDlg, idOldDefPushButton), WM_GETDLGCODE, 0, 0) & DLGC_DEFPUSHBUTTON) )
+           idOldDefPushButton =  GetDlgCtrlID(DEFDLG_FindDefButton( hwndDlg ));
+    if ( idOldDefPushButton &&
+         idOldDefPushButton != GetDlgCtrlID(hwndNew) )  /* There's already a default pushbutton */
     {
-        HWND hwndOld = GetDlgItem( hwndDlg, dlgInfo->idResult );
+        HWND hwndOld = GetDlgItem( hwndDlg, idOldDefPushButton );
         if (hwndOld && (SendMessageA( hwndOld, WM_GETDLGCODE, 0, 0) & DLGC_DEFPUSHBUTTON))
             SendMessageA( hwndOld, BM_SETSTYLE, BS_PUSHBUTTON, TRUE );
     }
     if (hwndNew)
     {
-        if(dlgcode==DLGC_UNDEFPUSHBUTTON)
+        if(dlgcode & DLGC_UNDEFPUSHBUTTON)
             SendMessageA( hwndNew, BM_SETSTYLE, BS_DEFPUSHBUTTON, TRUE );
     }
+
     return TRUE;
 }
 
@@ -214,7 +237,7 @@
 
         case DM_SETDEFID:
             if (dlgInfo && !(dlgInfo->flags & DF_END))
-                DEFDLG_SetDefButton( hwnd, dlgInfo, wParam );
+                DEFDLG_SetDefButton( hwnd, dlgInfo, wParam, TRUE );
             return 1;
 
         case DM_GETDEFID:
@@ -235,7 +258,7 @@
                 if (!lParam)
                     hwndDest = GetNextDlgTabItem(hwnd, GetFocus(), wParam);
                 if (hwndDest) DEFDLG_SetFocus( hwnd, hwndDest );
-                DEFDLG_SetDefButton( hwnd, dlgInfo, GetDlgCtrlID(hwndDest) );
+                DEFDLG_SetDefButton( hwnd, dlgInfo, GetDlgCtrlID(hwndDest), FALSE);
             }
             return 0;
 
